@@ -42,7 +42,7 @@ available.
 
     my $cwd = cwd();
 
-The cwd() is the most natural form for the current architecture. For
+The cwd() is the most natural form for the current architecture.  For
 most systems it is identical to `pwd` (but without the trailing line
 terminator).
 
@@ -57,9 +57,9 @@ chdir() you back into.  If fastcwd encounters a problem it will return
 undef but will probably leave you in a different directory.  For a
 measure of extra security, if everything appears to have worked, the
 fastcwd() function will check that it leaves you in the same directory
-that it started in. If it has changed it will C<die> with the message
+that it started in.  If it has changed it will C<die> with the message
 "Unstable directory path, current directory changed
-unexpectedly". That should never happen.
+unexpectedly".  That should never happen.
 
 =item fastgetcwd
 
@@ -129,15 +129,15 @@ it from Cwd.
 
 =item *
 
-Since the path seperators are different on some operating systems ('/'
+Since the path separators are different on some operating systems ('/'
 on Unix, ':' on MacPerl, etc...) we recommend you use the File::Spec
 modules wherever portability is a concern.
 
 =item *
 
 Actually, on Mac OS, the C<getcwd()>, C<fastgetcwd()> and C<fastcwd()>
-functions  are all aliases for the C<cwd()> function, which, on Mac OS,
-calls `pwd`. Likewise, the C<abs_path()> function is an alias for
+functions are all aliases for the C<cwd()> function, which, on Mac OS,
+calls `pwd`.  Likewise, the C<abs_path()> function is an alias for
 C<fast_abs_path()>.
 
 =back
@@ -171,9 +171,9 @@ use strict;
 use Exporter;
 use vars qw(@ISA @EXPORT @EXPORT_OK $VERSION);
 
-$VERSION = '3.32';
+$VERSION = '3.47';
 my $xs_version = $VERSION;
-$VERSION = eval $VERSION;
+$VERSION =~ tr/_//;
 
 @ISA = qw/ Exporter /;
 @EXPORT = qw(cwd getcwd fastcwd fastgetcwd);
@@ -242,19 +242,18 @@ sub _vms_efs {
 
 
 # If loading the XS stuff doesn't work, we can fall back to pure perl
-eval {
-  if ( $] >= 5.006 ) {
-    require XSLoader;
-    XSLoader::load( __PACKAGE__, $xs_version);
-  } else {
-    require DynaLoader;
-    push @ISA, 'DynaLoader';
-    __PACKAGE__->bootstrap( $xs_version );
-  }
-};
-
-# Must be after the DynaLoader stuff:
-$VERSION = eval $VERSION;
+unless (defined &getcwd) {
+  eval {
+    if ( $] >= 5.006 ) {
+      require XSLoader;
+      XSLoader::load( __PACKAGE__, $xs_version);
+    } else {
+      require DynaLoader;
+      push @ISA, 'DynaLoader';
+      __PACKAGE__->bootstrap( $xs_version );
+    }
+  };
+}
 
 # Big nasty table of function aliases
 my %METHOD_MAP =
@@ -344,9 +343,26 @@ foreach my $try ('/bin/pwd',
         last;
     }
 }
+
+# Android has a built-in pwd. Using $pwd_cmd will DTRT if
+# this perl was compiled with -Dd_useshellcmds, which is the
+# default for Android, but the block below is needed for the
+# miniperl running on the host when cross-compiling, and
+# potentially for native builds with -Ud_useshellcmds.
+if ($^O =~ /android/) {
+    # If targetsh is executable, then we're either a full
+    # perl, or a miniperl for a native build.
+    if (-x $Config::Config{targetsh}) {
+        $pwd_cmd = "$Config::Config{targetsh} -c pwd"
+    }
+    else {
+        $pwd_cmd = "$Config::Config{sh} -c pwd"
+    }
+}
+
 my $found_pwd_cmd = defined($pwd_cmd);
 unless ($pwd_cmd) {
-    # Isn't this wrong?  _backtick_pwd() will fail if somenone has
+    # Isn't this wrong?  _backtick_pwd() will fail if someone has
     # pwd in their path but it is not /bin/pwd or /usr/bin/pwd?
     # See [perl #16774]. --jhi
     $pwd_cmd = 'pwd';
@@ -582,6 +598,7 @@ sub _perl_abs_path
 	unless (opendir(PARENT, $dotdots))
 	{
 	    # probably a permissions issue.  Try the native command.
+	    require File::Spec;
 	    return File::Spec->rel2abs( $start, _backtick_pwd() );
 	}
 	unless (@cst = stat($dotdots))
@@ -626,8 +643,8 @@ sub fast_abs_path {
 
     # Detaint else we'll explode in taint mode.  This is safe because
     # we're not doing anything dangerous with it.
-    ($path) = $path =~ /(.*)/;
-    ($cwd)  = $cwd  =~ /(.*)/;
+    ($path) = $path =~ /(.*)/s;
+    ($cwd)  = $cwd  =~ /(.*)/s;
 
     unless (-e $path) {
  	_croak("$path: No such file or directory");
@@ -747,8 +764,22 @@ sub _os2_cwd {
     return $ENV{'PWD'};
 }
 
+sub _win32_cwd_simple {
+    $ENV{'PWD'} = `cd`;
+    chomp $ENV{'PWD'};
+    $ENV{'PWD'} =~ s:\\:/:g ;
+    return $ENV{'PWD'};
+}
+
 sub _win32_cwd {
-    if (eval 'defined &DynaLoader::boot_DynaLoader') {
+    # Need to avoid taking any sort of reference to the typeglob or the code in
+    # the optree, so that this tests the runtime state of things, as the
+    # ExtUtils::MakeMaker tests for "miniperl" need to be able to fake things at
+    # runtime by deleting the subroutine. *foo{THING} syntax on a symbol table
+    # lookup avoids needing a string eval, which has been reported to cause
+    # problems (for reasons that we haven't been able to get to the bottom of -
+    # rt.cpan.org #56225)
+    if (*{$DynaLoader::{boot_DynaLoader}}{CODE}) {
 	$ENV{'PWD'} = Win32::GetCwd();
     }
     else { # miniperl
@@ -758,7 +789,7 @@ sub _win32_cwd {
     return $ENV{'PWD'};
 }
 
-*_NT_cwd = defined &Win32::GetCwd ? \&_win32_cwd : \&_os2_cwd;
+*_NT_cwd = defined &Win32::GetCwd ? \&_win32_cwd : \&_win32_cwd_simple;
 
 sub _dos_cwd {
     if (!defined &Dos::GetCwd) {
